#!/bin/bash

path_itp = "path_to_itp_folder"
path_ts2cg = "path_to_TS2CG_folder"
path_mdp = "path_to_mdp_folder"

#################################################################
# PCG parameters
#################################################################

#small system
x=10
y=10
z=10

#no curvature
A=0
omega=0
pshift=0

#no wall
wall_density=0

#point density for lipid placement
density=2

#################################################################
#Create a flat bilayer
#################################################################

function make_bilayer_flat {
mkdir $system-FLAT
cd $system-FLAT

cp $path_ts2cg/input_template.str input-$system.str

sed -i "s/LIP1  lip1_up  lip1_down  lip1_apl/"$lip1" "$lip1_rat"  "$lip1_rat"  "$apl1"/" input-$system.str
sed -i "s/LIP2  lip2_up  lip2_down  lip2_apl/"$lip2" "$lip2_rat"  "$lip2_rat"  "$apl2"/" input-$system.str
sed -i "s/Box  X  Y  Z/Box "$x" "$y" "$z"/"  input-$system.str
sed -i "s/Density  dens_up  dens_down/Density "$density" "$density"/"  input-$system.str
sed -i "s/WallDensity  wall_up  wall_down/WallDensity "$wall_density" "$wall_density"/"  input-$system.str
sed -i "s/Thickness D/Thickness "$thickness"/"  input-$system.str
sed -i "s/Mode  amp  per  shift/Mode "$A" "$omega"  "$pshift"/"  input-$system.str

mkdir 1_PREP
cd 1_PREP

$path_ts2cg/PCG  -str ../input-$system.str  -Bondlength  0.25  -LLIB $path_ts2cg/Martini3.LIB -function analytical_shape

rm  topol.top
cat $path_itp/include.txt output.top >> topol.top

sed -i "s|path_to_itp|$path_itp|g" topol.top

cd ..

}

##############################################################
#energy minimize in vacuum
##############################################################

function EM_vac_nw {

mkdir 2_EM_vac
cd 2_EM_vac

gmx grompp -f $path_mdp/em_vacc_nw.mdp -c ../1_PREP/output.gro -p ../1_PREP/topol.top -r ../1_PREP/output.gro -o em_vacc.tpr -maxwarn 2
gmx mdrun -deffnm em_vacc -v

echo -e "keep 0 \n r ${lip1:0:4} | r ${lip2:0:4} \n name 1 Bilayer \n  q" |   gmx make_ndx -f em_vacc.gro

gmx grompp -f $path_mdp/nvt_vacc_nw.mdp -c em_vacc.gro -r em_vacc.gro -p ../1_PREP/topol.top -o nvt_vacc.tpr -n index.ndx
gmx mdrun -v -deffnm nvt_vacc -nb gpu -bonded gpu

cd ..
}

##################################################################
#Solvate & addition of ions
##################################################################

function solvate_nw {
mkdir 3_SOL
cd 3_SOL

$path_ts2cg/SOL -in ../2_EM_vac/nvt_vacc.gro -Rcutoff 0.32 -tem $path_ts2cg/water.gro -o sol.gro

calculate_ions
rm info.txt

$path_ts2cg/SOL -in ../2_EM_vac/nvt_vacc.gro -Rcutoff 0.32 -tem $path_ts2cg/water.gro -o sol.gro -ion $ions_p $ions_n #adjust
rm topol.top
cat ../1_PREP/topol.top  info.txt >> topol.top

gmx grompp -f $path_mdp/em_nw.mdp -c sol.gro -p topol.top -r sol.gro -o em.tpr -maxwarn 2
gmx mdrun -deffnm em -v

cd ..
}
#######################################################################
#equilibration
#######################################################################

function nvt_nw {
mkdir 4_EQ
cd 4_EQ

echo -e "keep 0 \n r ${lip1:0:4} | r ${lip2:0:4} \n name 1 Bilayer \n r W | a Na | a Cl \n name 2 W_ION \n q" |   gmx make_ndx -f ../3_SOL/sol.gro

gmx grompp -f $path_mdp/nvt_nw.mdp -c ../3_SOL/em.gro -r ../3_SOL/em.gro -p ../3_SOL/topol.top -o nvt.tpr -n index.ndx -maxwarn 1
gmx mdrun -v -deffnm nvt -nb gpu -bonded gpu -pin on
}

function npt_1_nw {
gmx grompp -f $path_mdp/npt_1_nw.mdp -c nvt.gro -r nvt.gro -p ../3_SOL/topol.top -o npt_1.tpr -n index.ndx -maxwarn 1
gmx mdrun -v -deffnm npt_1 -nb gpu -bonded gpu -pin on
}

function npt_2_nw {
gmx grompp -f $path_mdp/npt_2_nw.mdp -c npt_1.gro -r npt_1.gro -p ../3_SOL/topol.top -o npt_2.tpr -n index.ndx -maxwarn 1
gmx mdrun -v -deffnm npt_2 -nb gpu -bonded gpu -pin on
cd ..

mkdir 5_PROD
cd 5_PROD
}

#######################################################################
#Flat production run
#######################################################################

function RUN_flat {
gmx grompp -f $path_mdp/md_nw.mdp -c ../4_EQ/npt_2.gro -r ../4_EQ/npt_2.gro -p ../3_SOL/topol.top -n ../4_EQ/index.ndx -o md.tpr -maxwarn 1
gmx mdrun -v -deffnm md -pin on -nsteps 25000000 -nb gpu -bonded gpu

cd ../../

}

#######################################################################
#Calculate charged of systems
#######################################################################

function calculate_ions {

lip1_count="$(awk /$lip1/'{sum += $NF} END {print sum}' ../1_PREP/output.top)"
lip2_count="$(awk /$lip2/'{sum += $NF} END {print sum}' ../1_PREP/output.top)"
water_count="$(awk /W/'{water = $NF} END {print water}' ../3_SOL/info.txt)"
total_charge=$(($lip1_count * $charge_1 + $lip2_count * $charge_2))

ions=$(bc <<< " $water_count * 4 * $ion_conc / 55.08")

if [[ $total_charge -ge 0 ]]
	then ions_p=$ions
	else ions_p=$(($total_charge * -1 + $ions))
fi
if [[ $total_charge -le 0 ]]
	then ions_n=$ions
	else ions_n=$(($total_charge+$ions))
fi

}

#######################################################################
#Create & Run a flat system
#######################################################################

function flat_run {
	make_bilayer_flat
	EM_vac_nw
	solvate_nw
	nvt_nw
	npt_1_nw
	npt_2_nw
	RUN_flat

}

#######################################################################
#Analyze membrane thickness and area per lipid
#######################################################################

function analysis {

python3 flat_analysis.py $system-FLAT/5_PROD/

}

#######################################################################
#Do everything
#######################################################################

function full_run {
      system=$1-$2-$7-$8
      lip1=$1
      lip2=$2
      apl1=$3
      apl2=$4
      charge_1=$5
      charge_2=$6
      lip1_rat=$7
      lip2_rat=$8
      thickness=$9
      ion_conc=${10}
      flat_run
      analysis
}

#######################################################################
#Start a full run input Lipids and APLs and charges
#######################################################################

#		lip1   lip2    apl1    apl2 charge1 charge2  ratio1  ratio2 thickness bulk_ions
full_run	POPC 	CDL0	0.65	0.75	0	0	0.9	0.1	3.6     0.15
